@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #ifndef COMMAND_H_
@@ -16,6 +19,8 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include <cstdint>
 #include <string>
 
+class DataNode;
+
 
 
 // Class mapping key presses to specific commands / actions. The player can
@@ -41,6 +46,7 @@ public:
 	static const Command HAIL;
 	static const Command SCAN;
 	static const Command JUMP;
+	static const Command FLEET_JUMP;
 	static const Command TARGET;
 	static const Command NEAREST;
 	static const Command DEPLOY;
@@ -66,36 +72,39 @@ public:
 	// This command from the AI tells a ship that if possible, it should apply
 	// less than its full thrust in order to come to a complete stop.
 	static const Command STOP;
-	// Modifier command, usually triggered by shift-key. Changes behaviour of
+	// Modifier command, usually triggered by shift-key. Changes behavior of
 	// other commands like NEAREST, TARGET, HAIL and BOARD.
 	static const Command SHIFT;
 
-	
+
 public:
 	// In the given text, replace any instances of command names (in angle
 	// brackets) with key names (in quotes).
 	static std::string ReplaceNamesWithKeys(const std::string &text);
-	
+
 public:
 	Command() = default;
 	// Create a command representing whatever command is mapped to the given
 	// keycode (if any).
 	explicit Command(int keycode);
-	
+
 	// Read the current keyboard state and set this object to reflect it.
 	void ReadKeyboard();
-	
+
 	// Load or save the keyboard preferences.
 	static void LoadSettings(const std::string &path);
 	static void SaveSettings(const std::string &path);
 	static void SetKey(Command command, int keycode);
-	
+
 	// Get the description or keycode name for this command. If this command is
 	// a combination of more than one command, an empty string is returned.
 	const std::string &Description() const;
 	const std::string &KeyName() const;
 	bool HasConflict() const;
-	
+
+	// Load this command from an input file (for testing or scripted missions).
+	void Load(const DataNode &node);
+
 	// Reset this to an empty command.
 	void Clear();
 	// Clear, set, or check the given bits. This ignores the turn field.
@@ -106,46 +115,34 @@ public:
 	Command And(Command command) const;
 	// Get the commands that are set in this and not in the given command.
 	Command AndNot(Command command) const;
-	
+
 	// Get or set the turn amount. The amount must be between -1 and 1, but it
 	// can be a fractional value to allow finer control.
 	void SetTurn(double amount);
 	double Turn() const;
-	// Get or set the fire commands.
-	bool HasFire(int index) const;
-	void SetFire(int index);
-	// Check if any weapons are firing.
-	bool IsFiring() const;
-	// Set the turn rate of the turret with the given weapon index. A value of
-	// -1 or 1 means to turn at the full speed the turret is capable of.
-	double Aim(int index) const;
-	void SetAim(int index, double amount);
-	
+
 	// Check if any bits are set in this command (including a nonzero turn).
 	explicit operator bool() const;
 	bool operator!() const;
 	// This operator is just provided to allow commands to be used in a map.
 	bool operator<(const Command &command) const;
-	
+
 	// Get the commands that are set in either of these commands.
 	Command operator|(const Command &command) const;
 	Command &operator|=(const Command &command);
-	
-	
+
+
 private:
-	explicit Command(uint64_t state);
-	Command(uint64_t state, const std::string &text);
-	
-	
+	explicit Command(uint32_t state);
+	Command(uint32_t state, const std::string &text);
+
+
 private:
-	// The key commands and weapons to fire are stored in a single bitmask, with
-	// 32 bits for key commands and 32 bits for individual weapons.
-	// Ship::Load gives a soft warning for ships with more than 32 weapons.
-	uint64_t state = 0;
+	// The key commands are stored in a single bitmask with
+	// 32 bits for key commands.
+	uint32_t state = 0;
 	// Turning amount is stored as a separate double to allow fractional values.
 	double turn = 0.;
-	// Turret turn rates, reduced to 8 bits to save space.
-	signed char aim[32] = {};
 };
 
 
