@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "Depreciation.h"
@@ -49,19 +52,19 @@ void Depreciation::Load(const DataNode &node)
 	// Check if this is fleet or stock depreciation.
 	isStock = (node.Token(0) == NAME[1]);
 	isLoaded = true;
-	
+
 	for(const DataNode &child : node)
 	{
 		bool isShip = (child.Token(0) == "ship");
 		bool isOutfit = (child.Token(0) == "outfit");
 		if(!(isShip || isOutfit) || child.Size() < 2)
 			continue;
-		
+
 		// Figure out which record we're modifying.
 		map<int, int> &entry = isShip ?
 			ships[GameData::Ships().Get(child.Token(1))] :
 			outfits[GameData::Outfits().Get(child.Token(1))];
-		
+
 		// Load any depreciation records for this item.
 		for(const DataNode &grand : child)
 			if(grand.Size() >= 2)
@@ -77,32 +80,40 @@ void Depreciation::Save(DataWriter &out, int day) const
 	out.Write(NAME[isStock]);
 	out.BeginChild();
 	{
-		for(const auto &sit : ships)
-		{
-			out.Write("ship", sit.first->ModelName());
-			out.BeginChild();
+		using ShipElement = pair<const Ship *const, map<int, int>>;
+		WriteSorted(ships,
+			[](const ShipElement *lhs, const ShipElement *rhs)
+				{ return lhs->first->ModelName() < rhs->first->ModelName(); },
+			[=, &out](const ShipElement &sit)
 			{
-				// If this is a planet's stock, remember how many outfits in
-				// stock are fully depreciated. If it's the player's stock,
-				// anything not recorded is considered fully depreciated, so
-				// there is no reason to save records for those items.
-				for(const auto &it : sit.second)
-					if(isStock || (it.second && it.first > day - MAX_AGE))
-						out.Write(it.first, it.second);
-			}
-			out.EndChild();
-		}
-		for(const auto &oit : outfits)
-		{
-			out.Write("outfit", oit.first->Name());
-			out.BeginChild();
+				out.Write("ship", sit.first->ModelName());
+				out.BeginChild();
+				{
+					// If this is a planet's stock, remember how many outfits in
+					// stock are fully depreciated. If it's the player's stock,
+					// anything not recorded is considered fully depreciated, so
+					// there is no reason to save records for those items.
+					for(const auto &it : sit.second)
+						if(isStock || (it.second && it.first > day - MAX_AGE))
+							out.Write(it.first, it.second);
+				}
+				out.EndChild();
+			});
+		using OutfitElement = pair<const Outfit *const, map<int, int>>;
+		WriteSorted(outfits,
+			[](const OutfitElement *lhs, const OutfitElement *rhs)
+				{ return lhs->first->Name() < rhs->first->Name(); },
+			[=, &out](const OutfitElement &oit)
 			{
-				for(const auto &it : oit.second)
-					if(isStock || (it.second && it.first > day - MAX_AGE))
-						out.Write(it.first, it.second);
-			}
-			out.EndChild();
-		}
+				out.Write("outfit", oit.first->Name());
+				out.BeginChild();
+				{
+					for(const auto &it : oit.second)
+						if(isStock || (it.second && it.first > day - MAX_AGE))
+							out.Write(it.first, it.second);
+				}
+				out.EndChild();
+			});
 	}
 	out.EndChild();
 }
@@ -127,7 +138,7 @@ void Depreciation::Init(const vector<shared_ptr<Ship>> &fleet, int day)
 	{
 		const Ship *base = GameData::Ships().Get(ship->ModelName());
 		++ships[base][day];
-		
+
 		for(const auto &it : ship->Outfits())
 			outfits[it.first][day] += it.second;
 	}
@@ -142,7 +153,7 @@ void Depreciation::Buy(const Ship &ship, int day, Depreciation *source)
 	for(const auto &it : ship.Outfits())
 		for(int i = 0; i < it.second; ++i)
 			Buy(it.first, day, source);
-	
+
 	// Then, check the base day for the ship chassis itself.
 	const Ship *base = GameData::Ships().Get(ship.ModelName());
 	if(source)
@@ -162,7 +173,7 @@ void Depreciation::Buy(const Ship &ship, int day, Depreciation *source)
 			day -= MAX_AGE;
 		}
 	}
-	
+
 	// Increment our count for this ship on this day.
 	++ships[base][day];
 }
@@ -174,7 +185,7 @@ void Depreciation::Buy(const Outfit *outfit, int day, Depreciation *source)
 {
 	if(outfit->Get("installable") < 0.)
 		return;
-	
+
 	if(source)
 	{
 		// Check if the source has any instances of this outfit.
@@ -192,7 +203,7 @@ void Depreciation::Buy(const Outfit *outfit, int day, Depreciation *source)
 			day -= MAX_AGE;
 		}
 	}
-	
+
 	// Increment our count for this outfit on this day.
 	++outfits[outfit][day];
 }
@@ -204,16 +215,16 @@ int64_t Depreciation::Value(const vector<shared_ptr<Ship>> &fleet, int day) cons
 {
 	map<const Ship *, int> shipCount;
 	map<const Outfit *, int> outfitCount;
-	
+
 	for(const shared_ptr<Ship> &ship : fleet)
 	{
 		const Ship *base = GameData::Ships().Get(ship->ModelName());
 		++shipCount[base];
-		
+
 		for(const auto &it : ship->Outfits())
 			outfitCount[it.first] += it.second;
 	}
-	
+
 	int64_t value = 0;
 	for(const auto &it : shipCount)
 		value += Value(it.first, day, it.second);
@@ -244,7 +255,7 @@ int64_t Depreciation::Value(const Ship *ship, int day, int count) const
 	auto recordIt = ships.find(ship);
 	if(recordIt == ships.end() || recordIt->second.empty())
 		return DefaultDepreciation() * count * ship->ChassisCost();
-	
+
 	return Depreciate(recordIt->second, day, count) * ship->ChassisCost();
 }
 
@@ -255,13 +266,13 @@ int64_t Depreciation::Value(const Outfit *outfit, int day, int count) const
 {
 	if(outfit->Get("installable") < 0.)
 		return count * outfit->Cost();
-	
+
 	// Check whether a record exists for this outfit. If not, its value is full
 	// if this is  planet's stock, or fully depreciated if this is the player.
 	auto recordIt = outfits.find(outfit);
 	if(recordIt == outfits.end() || recordIt->second.empty())
 		return DefaultDepreciation() * count * outfit->Cost();
-	
+
 	return Depreciate(recordIt->second, day, count) * outfit->Cost();
 }
 
@@ -274,13 +285,13 @@ int Depreciation::Sell(map<int, int> &record)
 	// If we're a planet, we start by selling the oldest, cheapest thing.
 	auto it = (isStock ? record.begin() : --record.end());
 	int day = it->first;
-	
+
 	// Remove one record from the source. If necessary, delete this
 	// record line or the entire record for this outfit.
 	--it->second;
 	if(!it->second)
 		record.erase(it);
-	
+
 	return day;
 }
 
@@ -291,11 +302,11 @@ double Depreciation::Depreciate(const map<int, int> &record, int day, int count)
 {
 	if(record.empty())
 		return count * DefaultDepreciation();
-	
+
 	// Depending on whether this is a planet's stock or a player's fleet, we
 	// should either start with the oldest item, or the newest.
 	map<int, int>::const_iterator it = (isStock ? record.begin() : --record.end());
-	
+
 	double sum = 0.;
 	while(true)
 	{
@@ -308,7 +319,7 @@ double Depreciation::Depreciate(const map<int, int> &record, int day, int count)
 		// Bail out if we've counted enough items.
 		if(!count)
 			break;
-		
+
 		// Increment the iterator in the proper direction.
 		if(isStock)
 		{
@@ -336,7 +347,7 @@ double Depreciation::Depreciate(int age) const
 		return 1.;
 	if(age >= MAX_AGE)
 		return FULL_DEPRECIATION;
-	
+
 	double daily = pow(DAILY_DEPRECIATION, age - GRACE_PERIOD);
 	double linear = static_cast<double>(MAX_AGE - age) / (MAX_AGE - GRACE_PERIOD);
 	return FULL_DEPRECIATION + (1. - FULL_DEPRECIATION) * daily * linear;
