@@ -7,7 +7,10 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "DrawList.h"
@@ -47,25 +50,19 @@ void DrawList::SetCenter(const Point &center, const Point &centerVelocity)
 // Add an object based on the Body class.
 bool DrawList::Add(const Body &body, double cloak)
 {
-	Point position = body.Position() - center;
-	Point blur = body.Velocity() - centerVelocity;
-	if(Cull(body, position, blur) || cloak >= 1.)
-		return false;
-	
-	Push(body, position, blur, cloak, 1., body.GetSwizzle());
-	return true;
+	return Add(body, body.Position(), cloak);
 }
 
 
 
-bool DrawList::Add(const Body &body, Point position)
+bool DrawList::Add(const Body &body, Point position, double cloak)
 {
 	position -= center;
 	Point blur = body.Velocity() - centerVelocity;
 	if(Cull(body, position, blur))
 		return false;
-	
-	Push(body, position, blur, 0., 1., body.GetSwizzle());
+
+	Push(body, std::move(position), std::move(blur), cloak, body.GetSwizzle());
 	return true;
 }
 
@@ -77,8 +74,8 @@ bool DrawList::AddUnblurred(const Body &body)
 	Point blur;
 	if(Cull(body, position, blur))
 		return false;
-	
-	Push(body, position, blur, 0., 1., body.GetSwizzle());
+
+	Push(body, position, blur, 0., body.GetSwizzle());
 	return true;
 }
 
@@ -90,8 +87,8 @@ bool DrawList::AddSwizzled(const Body &body, int swizzle)
 	Point blur = body.Velocity() - centerVelocity;
 	if(Cull(body, position, blur))
 		return false;
-	
-	Push(body, position, blur, 0., 1., swizzle);
+
+	Push(body, position, blur, 0., swizzle);
 	return true;
 }
 
@@ -101,11 +98,11 @@ bool DrawList::AddSwizzled(const Body &body, int swizzle)
 void DrawList::Draw() const
 {
 	SpriteShader::Bind();
-	
+
 	bool withBlur = Preferences::Has("Render motion blur");
 	for(const SpriteShader::Item &item : items)
 		SpriteShader::Add(item, withBlur);
-	
+
 	SpriteShader::Unbind();
 }
 
@@ -115,7 +112,7 @@ bool DrawList::Cull(const Body &body, const Point &position, const Point &blur)
 {
 	if(!body.HasSprite() || !body.Zoom())
 		return true;
-	
+
 	Point unit = body.Facing().Unit();
 	// Cull sprites that are completely off screen, to reduce the number of draw
 	// calls that we issue (which may be the bottleneck on some systems).
@@ -128,38 +125,30 @@ bool DrawList::Cull(const Body &body, const Point &position, const Point &blur)
 		return true;
 	if(topLeft.X() > Screen::Right() || topLeft.Y() > Screen::Bottom())
 		return true;
-	
+
 	return false;
 }
 
 
 
-void DrawList::Push(const Body &body, Point pos, Point blur, double cloak, double clip, int swizzle)
+void DrawList::Push(const Body &body, Point pos, Point blur, double cloak, int swizzle)
 {
 	SpriteShader::Item item;
-	
+
 	item.texture = body.GetSprite()->Texture(isHighDPI);
 	item.frame = body.GetFrame(step);
 	item.frameCount = body.GetSprite()->Frames();
-	
+
+	item.position[0] = static_cast<float>(pos.X() * zoom);
+	item.position[1] = static_cast<float>(pos.Y() * zoom);
+
 	// Get unit vectors in the direction of the object's width and height.
 	double width = body.Width();
 	double height = body.Height();
 	Point unit = body.Facing().Unit();
 	Point uw = unit * width;
 	Point uh = unit * height;
-	
-	item.clip = clip;
-	if(clip < 1.)
-	{
-		// "clip" is the fraction of its height that we're clipping the sprite
-		// to. We still want it to start at the same spot, though.
-		pos -= uh * ((1. - clip) * .5);
-		uh *= clip;
-	}
-	item.position[0] = static_cast<float>(pos.X() * zoom);
-	item.position[1] = static_cast<float>(pos.Y() * zoom);
-	
+
 	// (0, -1) means a zero-degree rotation (since negative Y is up).
 	uw *= zoom;
 	uh *= zoom;
@@ -167,14 +156,15 @@ void DrawList::Push(const Body &body, Point pos, Point blur, double cloak, doubl
 	item.transform[1] = uw.X();
 	item.transform[2] = -uh.X();
 	item.transform[3] = -uh.Y();
-	
+
 	// Calculate the blur vector, in texture coordinates.
 	blur *= zoom;
 	item.blur[0] = unit.Cross(blur) / (width * 4.);
 	item.blur[1] = -unit.Dot(blur) / (height * 4.);
-	
+
 	item.alpha = 1. - cloak;
 	item.swizzle = swizzle;
-	
+	item.clip = 1.;
+
 	items.push_back(item);
 }
