@@ -7,31 +7,37 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "PreferencesPanel.h"
 
+#include "text/alignment.hpp"
 #include "Audio.h"
 #include "Color.h"
 #include "Dialog.h"
 #include "Files.h"
-#include "Font.h"
-#include "FontSet.h"
+#include "text/Font.h"
+#include "text/FontSet.h"
 #include "GameData.h"
 #include "Information.h"
 #include "Interface.h"
+#include "text/layout.hpp"
 #include "Preferences.h"
 #include "Screen.h"
 #include "Sprite.h"
 #include "SpriteSet.h"
 #include "SpriteShader.h"
 #include "StarField.h"
-#include "Table.h"
+#include "text/Table.h"
+#include "text/truncate.hpp"
 #include "UI.h"
-#include "WrappedText.h"
+#include "text/WrappedText.h"
 
-#include "gl_header.h"
+#include "opengl.h"
 #include <SDL2/SDL.h>
 
 #include <algorithm>
@@ -45,6 +51,8 @@ namespace {
 	const int ZOOM_FACTOR_MAX = 200;
 	const int ZOOM_FACTOR_INCREMENT = 10;
 	const string VIEW_ZOOM_FACTOR = "View zoom factor";
+	const string SCREEN_MODE_SETTING = "Screen mode";
+	const string VSYNC_SETTING = "VSync";
 	const string EXPEND_AMMO = "Escorts expend ammo";
 	const string TURRET_TRACKING = "Turret tracking";
 	const string FOCUS_PREFERENCE = "Turrets focus fire";
@@ -52,6 +60,7 @@ namespace {
 	const string REACTIVATE_HELP = "Reactivate first-time help";
 	const string SCROLL_SPEED = "Scroll speed";
 	const string FIGHTER_REPAIR = "Repair fighters in";
+	const string SHIP_OUTLINES = "Ship outlines in shops";
 }
 
 
@@ -61,7 +70,7 @@ PreferencesPanel::PreferencesPanel()
 {
 	if(!GameData::PluginAboutText().empty())
 		selectedPlugin = GameData::PluginAboutText().begin()->first;
-	
+
 	SetIsFullScreen(true);
 }
 
@@ -72,14 +81,14 @@ void PreferencesPanel::Draw()
 {
 	glClear(GL_COLOR_BUFFER_BIT);
 	GameData::Background().Draw(Point(), Point());
-	
+
 	Information info;
 	info.SetBar("volume", Audio::Volume());
 	GameData::Interfaces().Get("menu background")->Draw(info, this);
 	string pageName = (page == 'c' ? "controls" : page == 's' ? "settings" : "plugins");
 	GameData::Interfaces().Get(pageName)->Draw(info, this);
 	GameData::Interfaces().Get("preferences")->Draw(info, this);
-	
+
 	zones.clear();
 	prefZones.clear();
 	pluginZones.clear();
@@ -101,7 +110,7 @@ bool PreferencesPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &comma
 		EndEditing();
 		return true;
 	}
-	
+
 	if(key == SDLK_DOWN && static_cast<unsigned>(selected + 1) < zones.size())
 		++selected;
 	else if(key == SDLK_UP && selected > 0)
@@ -114,7 +123,7 @@ bool PreferencesPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &comma
 		page = key;
 	else
 		return false;
-	
+
 	return true;
 }
 
@@ -123,22 +132,24 @@ bool PreferencesPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &comma
 bool PreferencesPanel::Click(int x, int y, int clicks)
 {
 	EndEditing();
-	
+
 	if(x >= 265 && x < 295 && y >= -220 && y < 70)
 	{
 		Audio::SetVolume((20 - y) / 200.);
 		Audio::Play(Audio::Get("warder"));
 		return true;
 	}
-	
+
 	Point point(x, y);
 	for(unsigned index = 0; index < zones.size(); ++index)
 		if(zones[index].Contains(point))
 			editing = selected = index;
-	
+
 	for(const auto &zone : prefZones)
 		if(zone.Contains(point))
 		{
+			// For some settings, clicking the option does more than just toggle a
+			// boolean state keyed by the option's name.
 			if(zone.Value() == ZOOM_FACTOR)
 			{
 				int newZoom = Screen::UserZoom() + ZOOM_FACTOR_INCREMENT;
@@ -165,9 +176,15 @@ bool PreferencesPanel::Click(int x, int y, int clicks)
 				if(!Preferences::ZoomViewIn())
 					while(Preferences::ZoomViewOut()) {}
 			}
-			
-			// Update saved preferences.
-			if(zone.Value() == EXPEND_AMMO)
+			else if(zone.Value() == SCREEN_MODE_SETTING)
+				Preferences::ToggleScreenMode();
+			else if(zone.Value() == VSYNC_SETTING)
+			{
+				if(!Preferences::ToggleVSync())
+					GetUI()->Push(new Dialog(
+						"Unable to change VSync state. (Your system's graphics settings may be controlling it instead.)"));
+			}
+			else if(zone.Value() == EXPEND_AMMO)
 				Preferences::ToggleAmmoUsage();
 			else if(zone.Value() == TURRET_TRACKING)
 				Preferences::Set(FOCUS_PREFERENCE, !Preferences::Has(FOCUS_PREFERENCE));
@@ -178,24 +195,25 @@ bool PreferencesPanel::Click(int x, int y, int clicks)
 			}
 			else if(zone.Value() == SCROLL_SPEED)
 			{
-				// Toogle between three different speeds.
+				// Toggle between three different speeds.
 				int speed = Preferences::ScrollSpeed() + 20;
 				if(speed > 60)
 					speed = 20;
 				Preferences::SetScrollSpeed(speed);
 			}
+			// All other options are handled by just toggling the boolean state.
 			else
 				Preferences::Set(zone.Value(), !Preferences::Has(zone.Value()));
 			break;
 		}
-	
+
 	for(const auto &zone : pluginZones)
 		if(zone.Contains(point))
 		{
 			selectedPlugin = zone.Value();
 			break;
 		}
-	
+
 	return true;
 }
 
@@ -204,22 +222,22 @@ bool PreferencesPanel::Click(int x, int y, int clicks)
 bool PreferencesPanel::Hover(int x, int y)
 {
 	hoverPoint = Point(x, y);
-	
+
 	hover = -1;
 	for(unsigned index = 0; index < zones.size(); ++index)
 		if(zones[index].Contains(hoverPoint))
 			hover = index;
-	
+
 	hoverPreference.clear();
 	for(const auto &zone : prefZones)
 		if(zone.Contains(hoverPoint))
 			hoverPreference = zone.Value();
-	
+
 	hoverPlugin.clear();
 	for(const auto &zone : pluginZones)
 		if(zone.Contains(hoverPoint))
 			hoverPlugin = zone.Value();
-	
+
 	return true;
 }
 
@@ -230,7 +248,7 @@ bool PreferencesPanel::Scroll(double dx, double dy)
 {
 	if(!dy || hoverPreference.empty())
 		return false;
-	
+
 	if(hoverPreference == ZOOM_FACTOR)
 	{
 		int zoom = Screen::UserZoom();
@@ -238,11 +256,11 @@ bool PreferencesPanel::Scroll(double dx, double dy)
 			zoom -= ZOOM_FACTOR_INCREMENT;
 		if(dy > 0. && zoom < ZOOM_FACTOR_MAX)
 			zoom += ZOOM_FACTOR_INCREMENT;
-		
+
 		Screen::SetZoom(zoom);
 		if(Screen::Zoom() != zoom)
 			Screen::SetZoom(Screen::Zoom());
-		
+
 		// Convert to raw window coordinates, at the new zoom level.
 		Point point = hoverPoint * (Screen::Zoom() / 100.);
 		point += .5 * Point(Screen::RawWidth(), Screen::RawHeight());
@@ -282,18 +300,18 @@ void PreferencesPanel::DrawControls()
 	const Color &dim = *GameData::Colors().Get("dim");
 	const Color &medium = *GameData::Colors().Get("medium");
 	const Color &bright = *GameData::Colors().Get("bright");
-	
+
 	// Check for conflicts.
-	Color red(.3f, 0.f, 0.f, .3f);
-	
+	const Color &warning = *GameData::Colors().Get("warning conflict");
+
 	Table table;
-	table.AddColumn(-115, Table::LEFT);
-	table.AddColumn(115, Table::RIGHT);
+	table.AddColumn(-115, {230, Alignment::LEFT});
+	table.AddColumn(115, {230, Alignment::RIGHT});
 	table.SetUnderline(-120, 120);
-	
+
 	int firstY = -248;
 	table.DrawAt(Point(-130, firstY));
-	
+
 	static const string CATEGORIES[] = {
 		"Navigation",
 		"Weapons",
@@ -341,7 +359,7 @@ void PreferencesPanel::DrawControls()
 		// The "BREAK" line is where to go to the next column.
 		if(&command == BREAK)
 			table.DrawAt(Point(130, firstY));
-		
+
 		if(!command)
 		{
 			table.DrawGap(10);
@@ -362,9 +380,9 @@ void PreferencesPanel::DrawControls()
 			if(isConflicted || isEditing)
 			{
 				table.SetHighlight(56, 120);
-				table.DrawHighlight(isEditing ? dim: red);
+				table.DrawHighlight(isEditing ? dim : warning);
 			}
-			
+
 			// Mark the selected row.
 			bool isHovering = (index == hover && !isEditing);
 			if(!isHovering && index == selected)
@@ -372,24 +390,24 @@ void PreferencesPanel::DrawControls()
 				table.SetHighlight(-120, 54);
 				table.DrawHighlight(back);
 			}
-			
+
 			// Highlight whichever row the mouse hovers over.
 			table.SetHighlight(-120, 120);
 			if(isHovering)
 				table.DrawHighlight(back);
-			
+
 			zones.emplace_back(table.GetCenterPoint(), table.GetRowSize(), command);
-			
+
 			table.Draw(command.Description(), medium);
 			table.Draw(command.KeyName(), isEditing ? bright : medium);
 		}
 	}
-	
+
 	Table shiftTable;
-	shiftTable.AddColumn(125, Table::RIGHT);
+	shiftTable.AddColumn(125, {150, Alignment::RIGHT});
 	shiftTable.SetUnderline(0, 130);
 	shiftTable.DrawAt(Point(-400, 52));
-	
+
 	shiftTable.DrawUnderline(medium);
 	shiftTable.Draw("With <shift> key", bright);
 	shiftTable.DrawGap(5);
@@ -407,19 +425,21 @@ void PreferencesPanel::DrawSettings()
 	const Color &dim = *GameData::Colors().Get("dim");
 	const Color &medium = *GameData::Colors().Get("medium");
 	const Color &bright = *GameData::Colors().Get("bright");
-	
+
 	Table table;
-	table.AddColumn(-115, Table::LEFT);
-	table.AddColumn(115, Table::RIGHT);
+	table.AddColumn(-115, {230, Alignment::LEFT});
+	table.AddColumn(115, {230, Alignment::RIGHT});
 	table.SetUnderline(-120, 120);
-	
+
 	int firstY = -248;
 	table.DrawAt(Point(-130, firstY));
-	
+
 	static const string SETTINGS[] = {
 		"Display",
 		ZOOM_FACTOR,
 		VIEW_ZOOM_FACTOR,
+		SCREEN_MODE_SETTING,
+		VSYNC_SETTING,
 		"Show status overlays",
 		"Highlight player's flagship",
 		"Rotate flagship in HUD",
@@ -439,15 +459,20 @@ void PreferencesPanel::DrawSettings()
 		"Reduce large graphics",
 		"Draw background haze",
 		"Draw starfield",
+		"Parallax background",
 		"Show hyperspace flash",
+		SHIP_OUTLINES,
 		"",
 		"Other",
 		"Clickable radar display",
 		"Hide unexplored map regions",
 		REACTIVATE_HELP,
+		"Interrupt fast-forward",
 		"Rehire extra crew when lost",
 		SCROLL_SPEED,
 		"Show escort systems on map",
+		"Show stored outfits on map",
+		"System map sends move orders",
 		"Warning siren"
 	};
 	bool isCategory = true;
@@ -461,7 +486,7 @@ void PreferencesPanel::DrawSettings()
 				table.DrawAt(Point(130, firstY));
 			continue;
 		}
-		
+
 		if(isCategory)
 		{
 			isCategory = false;
@@ -472,10 +497,10 @@ void PreferencesPanel::DrawSettings()
 			table.DrawGap(5);
 			continue;
 		}
-		
+
 		// Record where this setting is displayed, so the user can click on it.
 		prefZones.emplace_back(table.GetCenterPoint(), table.GetRowSize(), setting);
-		
+
 		// Get the "on / off" text for this setting. Setting "isOn"
 		// draws the setting "bright" (i.e. the setting is active).
 		bool isOn = Preferences::Has(setting);
@@ -490,6 +515,16 @@ void PreferencesPanel::DrawSettings()
 			isOn = true;
 			text = to_string(static_cast<int>(100. * Preferences::ViewZoom()));
 		}
+		else if(setting == SCREEN_MODE_SETTING)
+		{
+			isOn = true;
+			text = Preferences::ScreenModeSetting();
+		}
+		else if(setting == VSYNC_SETTING)
+		{
+			text = Preferences::VSyncSetting();
+			isOn = text != "off";
+		}
 		else if(setting == EXPEND_AMMO)
 			text = Preferences::AmmoUsage();
 		else if(setting == TURRET_TRACKING)
@@ -502,6 +537,11 @@ void PreferencesPanel::DrawSettings()
 			isOn = true;
 			text = Preferences::Has(FIGHTER_REPAIR) ? "parallel" : "series";
 		}
+		else if(setting == SHIP_OUTLINES)
+		{
+			isOn = true;
+			text = Preferences::Has(SHIP_OUTLINES) ? "fancy" : "fast";
+		}
 		else if(setting == REACTIVATE_HELP)
 		{
 			// Check how many help messages have been displayed.
@@ -517,14 +557,14 @@ void PreferencesPanel::DrawSettings()
 				for(const string &str : SPECIAL_HELP)
 					if(it.first.find(str) == 0)
 						special = true;
-				
+
 				if(!special)
 				{
 					++total;
 					shown += Preferences::Has("help: " + it.first);
 				}
 			}
-			
+
 			if(shown)
 				text = to_string(shown) + " / " + to_string(total);
 			else
@@ -540,7 +580,7 @@ void PreferencesPanel::DrawSettings()
 		}
 		else
 			text = isOn ? "on" : "off";
-		
+
 		if(setting == hoverPreference)
 			table.DrawHighlight(back);
 		table.Draw(setting, isOn ? medium : dim);
@@ -555,28 +595,28 @@ void PreferencesPanel::DrawPlugins()
 	const Color &back = *GameData::Colors().Get("faint");
 	const Color &medium = *GameData::Colors().Get("medium");
 	const Color &bright = *GameData::Colors().Get("bright");
-	
+
+	const int MAX_TEXT_WIDTH = 230;
 	Table table;
-	table.AddColumn(-115, Table::LEFT);
+	table.AddColumn(-115, {MAX_TEXT_WIDTH, Truncate::MIDDLE});
 	table.SetUnderline(-120, 120);
-	
+
 	int firstY = -238;
 	table.DrawAt(Point(-130, firstY));
 	table.DrawUnderline(medium);
 	table.Draw("Installed plugins:", bright);
 	table.DrawGap(5);
-	
-	const int MAX_TEXT_WIDTH = 230;
+
 	const Font &font = FontSet::Get(14);
 	for(const auto &plugin : GameData::PluginAboutText())
 	{
 		pluginZones.emplace_back(table.GetCenterPoint(), table.GetRowSize(), plugin.first);
-		
+
 		bool isSelected = (plugin.first == selectedPlugin);
 		if(isSelected || plugin.first == hoverPlugin)
 			table.DrawHighlight(back);
-		table.Draw(font.TruncateMiddle(plugin.first, MAX_TEXT_WIDTH), isSelected ? bright : medium);
-		
+		table.Draw(plugin.first, isSelected ? bright : medium);
+
 		if(isSelected)
 		{
 			const Sprite *sprite = SpriteSet::Get(plugin.first);
@@ -587,7 +627,7 @@ void PreferencesPanel::DrawPlugins()
 				SpriteShader::Draw(sprite, center);
 				top.Y() += sprite->Height() + 10.;
 			}
-			
+
 			WrappedText wrap(font);
 			wrap.SetWrapWidth(MAX_TEXT_WIDTH);
 			static const string EMPTY = "(No description given.)";
@@ -602,6 +642,6 @@ void PreferencesPanel::DrawPlugins()
 void PreferencesPanel::Exit()
 {
 	Command::SaveSettings(Files::Config() + "keys.txt");
-	
+
 	GetUI()->Pop(this);
 }
