@@ -7,16 +7,22 @@ Foundation, either version 3 of the License, or (at your option) any later versi
 
 Endless Sky is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
-PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+PARTICULAR PURPOSE. See the GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along with
+this program. If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include "ShopPanel.h"
 
+#include "text/alignment.hpp"
+#include "CategoryTypes.h"
 #include "Color.h"
+#include "text/DisplayText.h"
 #include "FillShader.h"
-#include "Font.h"
-#include "FontSet.h"
-#include "Format.h"
+#include "text/Font.h"
+#include "text/FontSet.h"
+#include "text/Format.h"
 #include "GameData.h"
 #include "Government.h"
 #include "OutlineShader.h"
@@ -30,10 +36,11 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "Sprite.h"
 #include "SpriteSet.h"
 #include "SpriteShader.h"
+#include "text/truncate.hpp"
 #include "UI.h"
-#include "WrappedText.h"
+#include "text/WrappedText.h"
 
-#include "gl_header.h"
+#include "opengl.h"
 #include <SDL2/SDL.h>
 
 #include <algorithm>
@@ -41,10 +48,12 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 using namespace std;
 
 namespace {
+	const string SHIP_OUTLINES = "Ship outlines in shops";
+
 	constexpr int ICON_TILE = 62;
 	constexpr int ICON_COLS = 4;
 	constexpr float ICON_SIZE = ICON_TILE - 8;
-	
+
 	bool CanShowInSidebar(const Ship &ship, const System *here)
 	{
 		return ship.GetSystem() == here && !ship.IsDisabled();
@@ -56,7 +65,7 @@ namespace {
 ShopPanel::ShopPanel(PlayerInfo &player, bool isOutfitter)
 	: player(player), day(player.GetDate().DaysSinceEpoch()),
 	planet(player.GetPlanet()), playerShip(player.Flagship()),
-	categories(isOutfitter ? Outfit::CATEGORIES : Ship::CATEGORIES),
+	categories(GameData::Category(isOutfitter ? CategoryType::OUTFIT : CategoryType::SHIP)),
 	collapsed(player.Collapsed(isOutfitter ? "outfitter" : "shipyard"))
 {
 	if(playerShip)
@@ -83,7 +92,8 @@ void ShopPanel::Step()
 		if(selectedBottomY > mainBottomY)
 			DoScroll(max(-30., mainBottomY - selectedBottomY));
 		// Scroll down until the bottoms or the tops match.
-		else if(selectedBottomY < mainBottomY && (mainBottomY - mainTopY < selectedBottomY - selectedTopY && selectedTopY < mainTopY))
+		else if(selectedBottomY < mainBottomY
+			&& (mainBottomY - mainTopY < selectedBottomY - selectedTopY && selectedTopY < mainTopY))
 			DoScroll(min(30., min(mainTopY - selectedTopY, mainBottomY - selectedBottomY)));
 		// Details are in view.
 		else
@@ -96,21 +106,22 @@ void ShopPanel::Step()
 void ShopPanel::Draw()
 {
 	const double oldSelectedTopY = selectedTopY;
-	
+
 	glClear(GL_COLOR_BUFFER_BIT);
-	
+
 	// Clear the list of clickable zones.
 	zones.clear();
 	categoryZones.clear();
-	
-	DrawSidebar();
+
+	DrawShipsSidebar();
+	DrawDetailsSidebar();
 	DrawButtons();
 	DrawMain();
 	DrawKey();
-	
+
 	shipInfo.DrawTooltips();
 	outfitInfo.DrawTooltips();
-	
+
 	if(!warningType.empty())
 	{
 		constexpr int WIDTH = 250;
@@ -119,24 +130,33 @@ void ShopPanel::Draw()
 		WrappedText wrap(FontSet::Get(14));
 		wrap.SetWrapWidth(WIDTH - 2 * PAD);
 		wrap.Wrap(text);
-		
+
 		bool isError = (warningType.back() == '!');
 		const Color &textColor = *GameData::Colors().Get("medium");
 		const Color &backColor = *GameData::Colors().Get(isError ? "error back" : "warning back");
-		
+
 		Point size(WIDTH, wrap.Height() + 2 * PAD);
 		Point anchor = Point(warningPoint.X(), min<double>(warningPoint.Y() + size.Y(), Screen::Bottom()));
 		FillShader::Fill(anchor - .5 * size, size, backColor);
 		wrap.Draw(anchor - size + Point(PAD, PAD), textColor);
 	}
-	
-	if(dragShip && dragShip->GetSprite())
+
+	if(dragShip && isDraggingShip && dragShip->GetSprite())
 	{
-		static const Color selected(.8f, 1.f);
 		const Sprite *sprite = dragShip->GetSprite();
 		float scale = ICON_SIZE / max(sprite->Width(), sprite->Height());
-		Point size(sprite->Width() * scale, sprite->Height() * scale);
-		OutlineShader::Draw(sprite, dragPoint, size, selected);
+		if(Preferences::Has(SHIP_OUTLINES))
+		{
+			static const Color selected(.8f, 1.f);
+			Point size(sprite->Width() * scale, sprite->Height() * scale);
+			OutlineShader::Draw(sprite, dragPoint, size, selected);
+		}
+		else
+		{
+			int swizzle = dragShip->CustomSwizzle() >= 0
+				? dragShip->CustomSwizzle() : GameData::PlayerGovernment()->GetSwizzle();
+			SpriteShader::Draw(sprite, dragPoint, scale, swizzle);
+		}
 	}
 
 	if(sameSelectedTopY)
@@ -154,47 +174,45 @@ void ShopPanel::Draw()
 
 
 
-void ShopPanel::DrawSidebar()
+void ShopPanel::DrawShipsSidebar()
 {
 	const Font &font = FontSet::Get(14);
 	const Color &medium = *GameData::Colors().Get("medium");
 	const Color &bright = *GameData::Colors().Get("bright");
 	sideDetailHeight = 0;
-	
+
 	// Fill in the background.
 	FillShader::Fill(
-		Point(Screen::Right() - SIDE_WIDTH / 2, 0.),
-		Point(SIDE_WIDTH, Screen::Height()),
+		Point(Screen::Right() - SIDEBAR_WIDTH / 2, 0.),
+		Point(SIDEBAR_WIDTH, Screen::Height()),
 		*GameData::Colors().Get("panel background"));
 	FillShader::Fill(
-		Point(Screen::Right() - SIDE_WIDTH, 0.),
+		Point(Screen::Right() - SIDEBAR_WIDTH, 0.),
 		Point(1, Screen::Height()),
 		*GameData::Colors().Get("shop side panel background"));
-	
+
 	// Draw this string, centered in the side panel:
 	static const string YOURS = "Your Ships:";
-	Point yoursPoint(
-		Screen::Right() - SIDE_WIDTH / 2 - font.Width(YOURS) / 2,
-		Screen::Top() + 10 - sideScroll);
-	font.Draw(YOURS, yoursPoint, bright);
-	
+	Point yoursPoint(Screen::Right() - SIDEBAR_WIDTH, Screen::Top() + 10 - sidebarScroll);
+	font.Draw({YOURS, {SIDEBAR_WIDTH, Alignment::CENTER}}, yoursPoint, bright);
+
 	// Start below the "Your Ships" label, and draw them.
 	Point point(
-		Screen::Right() - SIDE_WIDTH / 2 - 93,
-		Screen::Top() + SIDE_WIDTH / 2 - sideScroll + 40 - 93);
-	
+		Screen::Right() - SIDEBAR_WIDTH / 2 - 93,
+		Screen::Top() + SIDEBAR_WIDTH / 2 - sidebarScroll + 40 - 93);
+
 	const System *here = player.GetSystem();
 	int shipsHere = 0;
 	for(const shared_ptr<Ship> &ship : player.Ships())
 		shipsHere += CanShowInSidebar(*ship, here);
 	if(shipsHere < 4)
 		point.X() += .5 * ICON_TILE * (4 - shipsHere);
-	
+
 	// Check whether flight check tooltips should be shown.
 	const auto flightChecks = player.FlightCheck();
 	Point mouse = GetUI()->GetMouse();
 	warningType.clear();
-	
+
 	static const Color selected(.8f, 1.f);
 	static const Color unselected(.4f, 1.f);
 	for(const shared_ptr<Ship> &ship : player.Ships())
@@ -202,13 +220,13 @@ void ShopPanel::DrawSidebar()
 		// Skip any ships that are "absent" for whatever reason.
 		if(!CanShowInSidebar(*ship, here))
 			continue;
-	
+
 		if(point.X() > Screen::Right())
 		{
 			point.X() -= ICON_TILE * ICON_COLS;
 			point.Y() += ICON_TILE;
 		}
-		
+
 		bool isSelected = playerShips.count(ship.get());
 		const Sprite *background = SpriteSet::Get(isSelected ? "ui/icon selected" : "ui/icon unselected");
 		SpriteShader::Draw(background, point);
@@ -216,17 +234,25 @@ void ShopPanel::DrawSidebar()
 		// button (if any) applies to it. If so, brighten the background.
 		if(isSelected && ShouldHighlight(ship.get()))
 			SpriteShader::Draw(background, point);
-		
+
 		const Sprite *sprite = ship->GetSprite();
 		if(sprite)
 		{
 			float scale = ICON_SIZE / max(sprite->Width(), sprite->Height());
-			Point size(sprite->Width() * scale, sprite->Height() * scale);
-			OutlineShader::Draw(sprite, point, size, isSelected ? selected : unselected);
+			if(Preferences::Has(SHIP_OUTLINES))
+			{
+				Point size(sprite->Width() * scale, sprite->Height() * scale);
+				OutlineShader::Draw(sprite, point, size, isSelected ? selected : unselected);
+			}
+			else
+			{
+				int swizzle = ship->CustomSwizzle() >= 0 ? ship->CustomSwizzle() : GameData::PlayerGovernment()->GetSwizzle();
+				SpriteShader::Draw(sprite, point, scale, swizzle);
+			}
 		}
-		
+
 		zones.emplace_back(point, Point(ICON_TILE, ICON_TILE), ship.get());
-		
+
 		const auto checkIt = flightChecks.find(ship);
 		if(checkIt != flightChecks.end())
 		{
@@ -239,37 +265,66 @@ void ShopPanel::DrawSidebar()
 				warningPoint = zones.back().TopLeft();
 			}
 		}
-		
+
 		point.X() += ICON_TILE;
 	}
 	point.Y() += ICON_TILE;
-	
+
 	if(playerShip)
 	{
 		point.Y() += SHIP_SIZE / 2;
-		point.X() = Screen::Right() - SIDE_WIDTH / 2;
+		point.X() = Screen::Right() - SIDEBAR_WIDTH / 2;
 		DrawShip(*playerShip, point, true);
-		
-		Point offset(SIDE_WIDTH / -2, SHIP_SIZE / 2);
+
+		Point offset(SIDEBAR_WIDTH / -2, SHIP_SIZE / 2);
 		sideDetailHeight = DrawPlayerShipInfo(point + offset);
 		point.Y() += sideDetailHeight + SHIP_SIZE / 2;
 	}
 	else if(player.Cargo().Size())
 	{
-		point.X() = Screen::Right() - SIDE_WIDTH + 10;
+		point.X() = Screen::Right() - SIDEBAR_WIDTH + 10;
 		font.Draw("cargo space:", point, medium);
-		
+
 		string space = Format::Number(player.Cargo().Free()) + " / " + Format::Number(player.Cargo().Size());
-		Point right(Screen::Right() - font.Width(space) - 10, point.Y());
-		font.Draw(space, right, bright);
+		font.Draw({space, {SIDEBAR_WIDTH - 20, Alignment::RIGHT}}, point, bright);
 		point.Y() += 20.;
 	}
-	maxSideScroll = max(0., point.Y() + sideScroll - Screen::Bottom() + BUTTON_HEIGHT);
-	
+	maxSidebarScroll = max(0., point.Y() + sidebarScroll - Screen::Bottom() + BUTTON_HEIGHT);
+
 	PointerShader::Draw(Point(Screen::Right() - 10, Screen::Top() + 10),
-		Point(0., -1.), 10.f, 10.f, 5.f, Color(sideScroll > 0 ? .8f : .2f, 0.f));
+		Point(0., -1.), 10.f, 10.f, 5.f, Color(sidebarScroll > 0 ? .8f : .2f, 0.f));
 	PointerShader::Draw(Point(Screen::Right() - 10, Screen::Bottom() - 80),
-		Point(0., 1.), 10.f, 10.f, 5.f, Color(sideScroll < maxSideScroll ? .8f : .2f, 0.f));
+		Point(0., 1.), 10.f, 10.f, 5.f, Color(sidebarScroll < maxSidebarScroll ? .8f : .2f, 0.f));
+}
+
+
+
+void ShopPanel::DrawDetailsSidebar()
+{
+	// Fill in the background.
+	const Color &line = *GameData::Colors().Get("dim");
+	const Color &back = *GameData::Colors().Get("shop info panel background");
+	FillShader::Fill(
+		Point(Screen::Right() - SIDEBAR_WIDTH - INFOBAR_WIDTH, 0.),
+		Point(1., Screen::Height()),
+		line);
+	FillShader::Fill(
+		Point(Screen::Right() - SIDEBAR_WIDTH - INFOBAR_WIDTH / 2, 0.),
+		Point(INFOBAR_WIDTH - 1., Screen::Height()),
+		back);
+
+	Point point(
+		Screen::Right() - SIDE_WIDTH + INFOBAR_WIDTH / 2,
+		Screen::Top() + 10 - infobarScroll);
+
+	int heightOffset = DrawDetails(point);
+
+	maxInfobarScroll = max(0., heightOffset + infobarScroll - Screen::Bottom());
+
+	PointerShader::Draw(Point(Screen::Right() - SIDEBAR_WIDTH - 10, Screen::Top() + 10),
+		Point(0., -1.), 10.f, 10.f, 5.f, Color(infobarScroll > 0 ? .8f : .2f, 0.f));
+	PointerShader::Draw(Point(Screen::Right() - SIDEBAR_WIDTH - 10, Screen::Bottom() - 10),
+		Point(0., 1.), 10.f, 10.f, 5.f, Color(infobarScroll < maxInfobarScroll ? .8f : .2f, 0.f));
 }
 
 
@@ -277,52 +332,52 @@ void ShopPanel::DrawSidebar()
 void ShopPanel::DrawButtons()
 {
 	// The last 70 pixels on the end of the side panel are for the buttons:
-	Point buttonSize(SIDE_WIDTH, BUTTON_HEIGHT);
-	FillShader::Fill(Screen::BottomRight() - .5 * buttonSize, buttonSize, *GameData::Colors().Get("shop side panel background"));
+	Point buttonSize(SIDEBAR_WIDTH, BUTTON_HEIGHT);
+	FillShader::Fill(Screen::BottomRight() - .5 * buttonSize, buttonSize,
+		*GameData::Colors().Get("shop side panel background"));
 	FillShader::Fill(
-		Point(Screen::Right() - SIDE_WIDTH / 2, Screen::Bottom() - BUTTON_HEIGHT),
-		Point(SIDE_WIDTH, 1), *GameData::Colors().Get("shop side panel footer"));
-	
+		Point(Screen::Right() - SIDEBAR_WIDTH / 2, Screen::Bottom() - BUTTON_HEIGHT),
+		Point(SIDEBAR_WIDTH, 1), *GameData::Colors().Get("shop side panel footer"));
+
 	const Font &font = FontSet::Get(14);
 	const Color &bright = *GameData::Colors().Get("bright");
 	const Color &dim = *GameData::Colors().Get("medium");
 	const Color &back = *GameData::Colors().Get("panel background");
-	
-	Point point(
-		Screen::Right() - SIDE_WIDTH + 10,
+
+	const Point creditsPoint(
+		Screen::Right() - SIDEBAR_WIDTH + 10,
 		Screen::Bottom() - 65);
-	font.Draw("You have:", point, dim);
-	
-	string credits = Format::Credits(player.Accounts().Credits()) + " credits";
-	point.X() += (SIDE_WIDTH - 20) - font.Width(credits);
-	font.Draw(credits, point, bright);
-	
+	font.Draw("You have:", creditsPoint, dim);
+
+	const auto credits = Format::Credits(player.Accounts().Credits()) + " credits";
+	font.Draw({credits, {SIDEBAR_WIDTH - 20, Alignment::RIGHT}}, creditsPoint, bright);
+
 	const Font &bigFont = FontSet::Get(18);
 	const Color &hover = *GameData::Colors().Get("hover");
 	const Color &active = *GameData::Colors().Get("active");
 	const Color &inactive = *GameData::Colors().Get("inactive");
-	
-	Point buyCenter = Screen::BottomRight() - Point(210, 25);
+
+	const Point buyCenter = Screen::BottomRight() - Point(210, 25);
 	FillShader::Fill(buyCenter, Point(60, 30), back);
-	string BUY = (playerShip && selectedOutfit && player.Cargo().Get(selectedOutfit)) ? "_Install" : "_Buy";
+	string BUY = IsAlreadyOwned() ? (playerShip ? "_Install" : "_Cargo") : "_Buy";
 	bigFont.Draw(BUY,
 		buyCenter - .5 * Point(bigFont.Width(BUY), bigFont.Height()),
 		CanBuy() ? hoverButton == 'b' ? hover : active : inactive);
-	
-	Point sellCenter = Screen::BottomRight() - Point(130, 25);
+
+	const Point sellCenter = Screen::BottomRight() - Point(130, 25);
 	FillShader::Fill(sellCenter, Point(60, 30), back);
 	static const string SELL = "_Sell";
 	bigFont.Draw(SELL,
 		sellCenter - .5 * Point(bigFont.Width(SELL), bigFont.Height()),
 		CanSell() ? hoverButton == 's' ? hover : active : inactive);
-	
-	Point leaveCenter = Screen::BottomRight() - Point(45, 25);
+
+	const Point leaveCenter = Screen::BottomRight() - Point(45, 25);
 	FillShader::Fill(leaveCenter, Point(70, 30), back);
 	static const string LEAVE = "_Leave";
 	bigFont.Draw(LEAVE,
 		leaveCenter - .5 * Point(bigFont.Width(LEAVE), bigFont.Height()),
 		hoverButton == 'l' ? hover : active);
-	
+
 	int modifier = Modifier();
 	if(modifier > 1)
 	{
@@ -330,7 +385,7 @@ void ShopPanel::DrawButtons()
 		int modWidth = font.Width(mod);
 		font.Draw(mod, buyCenter + Point(-.5 * modWidth, 10.), dim);
 		if(CanSellMultiple())
-			font.Draw(mod, sellCenter + Point(-.5 * modWidth, 10.), dim);	
+			font.Draw(mod, sellCenter + Point(-.5 * modWidth, 10.), dim);
 	}
 }
 
@@ -342,10 +397,10 @@ void ShopPanel::DrawMain()
 	const Color &dim = *GameData::Colors().Get("medium");
 	const Color &bright = *GameData::Colors().Get("bright");
 	mainDetailHeight = 0;
-	
+
 	const Sprite *collapsedArrow = SpriteSet::Get("ui/collapsed");
 	const Sprite *expandedArrow = SpriteSet::Get("ui/expanded");
-	
+
 	// Draw all the available items.
 	// First, figure out how many columns we can draw.
 	const int TILE_SIZE = TileSize();
@@ -355,7 +410,7 @@ void ShopPanel::DrawMain()
 		return;
 	const int columns = mainWidth / TILE_SIZE;
 	const int columnWidth = mainWidth / columns;
-	
+
 	const Point begin(
 		(Screen::Width() - columnWidth) / -2,
 		(Screen::Height() - TILE_SIZE) / -2 - mainScroll);
@@ -368,59 +423,34 @@ void ShopPanel::DrawMain()
 		map<string, set<string>>::const_iterator it = catalog.find(category);
 		if(it == catalog.end())
 			continue;
-		
+
 		// This should never happen, but bail out if we don't know what planet
 		// we are on (meaning there's no way to know what items are for sale).
 		if(!planet)
 			break;
-		
+
 		Point side(Screen::Left() + 5., point.Y() - TILE_SIZE / 2 + 10);
 		point.Y() += bigFont.Height() + 20;
 		nextY += bigFont.Height() + 20;
-		
+
 		bool isCollapsed = collapsed.count(category);
 		bool isEmpty = true;
 		for(const string &name : it->second)
 		{
 			bool isSelected = (selectedShip && GameData::Ships().Get(name) == selectedShip)
 				|| (selectedOutfit && GameData::Outfits().Get(name) == selectedOutfit);
-			
+
 			if(isSelected)
 				selectedTopY = point.Y() - TILE_SIZE / 2;
-			
+
 			if(!HasItem(name))
 				continue;
 			isEmpty = false;
 			if(isCollapsed)
 				break;
-			
+
 			DrawItem(name, point, scrollY);
-			
-			if(isSelected)
-			{
-				Color color(.2f, 0.f);
-				int dy = DividerOffset();
-				
-				float before = point.X() - TILE_SIZE / 2 - Screen::Left();
-				FillShader::Fill(Point(Screen::Left() + .5f * before, point.Y() + dy),
-					Point(before, 1.), color);
-				
-				float after = endX - (static_cast<float>(point.X()) + TILE_SIZE / 2);
-				FillShader::Fill(Point(endX - .5f * after, point.Y() + dy),
-					Point(after, 1.), color);
-				
-				// The center of the display needs to be between these two values:
-				int panelAndAHalf = DetailWidth() / 2;
-				double minX = Screen::Left() + panelAndAHalf;
-				double maxX = Screen::Left() + mainWidth - panelAndAHalf;
-				Point center(
-					max(minX, min(maxX, point.X())),
-					point.Y() + TILE_SIZE / 2);
-				
-				mainDetailHeight = DrawDetails(center);
-				nextY += mainDetailHeight;
-			}
-			
+
 			point.X() += columnWidth;
 			if(point.X() >= endX)
 			{
@@ -430,14 +460,14 @@ void ShopPanel::DrawMain()
 				scrollY = -mainDetailHeight;
 			}
 		}
-		
+
 		if(!isEmpty)
 		{
 			Point size(bigFont.Width(category) + 25., bigFont.Height());
 			categoryZones.emplace_back(Point(Screen::Left(), side.Y()) + .5 * size, size, category);
 			SpriteShader::Draw(isCollapsed ? collapsedArrow : expandedArrow, side + Point(10., 10.));
 			bigFont.Draw(category, side + Point(25., 0.), isCollapsed ? dim : bright);
-			
+
 			if(point.X() != begin.X())
 			{
 				point.X() = begin.X();
@@ -456,11 +486,11 @@ void ShopPanel::DrawMain()
 	}
 	// This is how much Y space was actually used.
 	nextY -= 40 + TILE_SIZE;
-	
+
 	// What amount would mainScroll have to equal to make nextY equal the
 	// bottom of the screen? (Also leave space for the "key" at the bottom.)
-	maxMainScroll = max(0., nextY + mainScroll - Screen::Height() / 2 - TILE_SIZE / 2 + 40.);
-	
+	maxMainScroll = max(0., nextY + mainScroll - Screen::Height() / 2 - TILE_SIZE / 2 + VisibilityCheckboxesSize() + 40.);
+
 	PointerShader::Draw(Point(Screen::Right() - 10 - SIDE_WIDTH, Screen::Top() + 10),
 		Point(0., -1.), 10.f, 10.f, 5.f, Color(mainScroll > 0 ? .8f : .2f, 0.f));
 	PointerShader::Draw(Point(Screen::Right() - 10 - SIDE_WIDTH, Screen::Bottom() - 10),
@@ -474,13 +504,7 @@ void ShopPanel::DrawShip(const Ship &ship, const Point &center, bool isSelected)
 	const Sprite *back = SpriteSet::Get(
 		isSelected ? "ui/shipyard selected" : "ui/shipyard unselected");
 	SpriteShader::Draw(back, center);
-	
-	// Draw the ship name.
-	const Font &font = FontSet::Get(14);
-	const string &name = ship.Name().empty() ? ship.ModelName() : font.TruncateMiddle(ship.Name(), SIDE_WIDTH - 61);
-	Point offset(-.5f * font.Width(name), -.5f * SHIP_SIZE + 10.f);
-	font.Draw(name, center + offset, *GameData::Colors().Get("bright"));
-	
+
 	const Sprite *thumbnail = ship.Thumbnail();
 	const Sprite *sprite = ship.GetSprite();
 	int swizzle = ship.CustomSwizzle() >= 0 ? ship.CustomSwizzle() : GameData::PlayerGovernment()->GetSwizzle();
@@ -493,11 +517,18 @@ void ShopPanel::DrawShip(const Ship &ship, const Point &center, bool isSelected)
 		float zoom = min(1.f, zoomSize / max(sprite->Width(), sprite->Height()));
 		SpriteShader::Draw(sprite, center, zoom, swizzle);
 	}
+
+	// Draw the ship name.
+	const Font &font = FontSet::Get(14);
+	const string &name = ship.Name().empty() ? ship.ModelName() : ship.Name();
+	Point offset(-SIDEBAR_WIDTH / 2, -.5f * SHIP_SIZE + 10.f);
+	font.Draw({name, {SIDEBAR_WIDTH, Alignment::CENTER, Truncate::MIDDLE}},
+		center + offset, *GameData::Colors().Get("bright"));
 }
 
 
 
-void ShopPanel::FailSell(bool toCargo) const
+void ShopPanel::FailSell(bool toStorage) const
 {
 }
 
@@ -510,6 +541,20 @@ bool ShopPanel::CanSellMultiple() const
 
 
 
+// Helper function for UI buttons to determine if the selected item is
+// already owned. Affects if "Install" is shown for already owned items
+// or if "Buy" is shown for items not yet owned.
+//
+// If we are buying into cargo, then items in cargo don't count as already
+// owned, but they count as "already installed" in cargo.
+bool ShopPanel::IsAlreadyOwned() const
+{
+	return (playerShip && selectedOutfit && player.Cargo().Get(selectedOutfit))
+		|| (player.Storage() && player.Storage()->Get(selectedOutfit));
+}
+
+
+
 bool ShopPanel::ShouldHighlight(const Ship *ship)
 {
 	return (hoverButton == 's');
@@ -523,6 +568,13 @@ void ShopPanel::DrawKey()
 
 
 
+int ShopPanel::VisibilityCheckboxesSize() const
+{
+	return 0;
+}
+
+
+
 void ShopPanel::ToggleForSale()
 {
 	sameSelectedTopY = true;
@@ -530,6 +582,13 @@ void ShopPanel::ToggleForSale()
 
 
 
+void ShopPanel::ToggleStorage()
+{
+	sameSelectedTopY = true;
+}
+
+
+
 void ShopPanel::ToggleCargo()
 {
 	sameSelectedTopY = true;
@@ -541,38 +600,39 @@ void ShopPanel::ToggleCargo()
 bool ShopPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, bool isNewPress)
 {
 	scrollDetailsIntoView = false;
-	bool toCargo = selectedOutfit && (key == 'r' || key == 'u');
+	bool toStorage = selectedOutfit && (key == 'r' || key == 'u');
 	if(key == 'l' || key == 'd' || key == SDLK_ESCAPE
 			|| (key == 'w' && (mod & (KMOD_CTRL | KMOD_GUI))))
 	{
 		player.UpdateCargoCapacities();
 		GetUI()->Pop(this);
 	}
-	else if(key == 'b' || (key == 'i' && selectedOutfit && player.Cargo().Get(selectedOutfit)))
+	else if(key == 'b' || ((key == 'i' || key == 'c') && selectedOutfit && (player.Cargo().Get(selectedOutfit)
+			|| (player.Storage() && player.Storage()->Get(selectedOutfit)))))
 	{
-		if(!CanBuy())
+		if(!CanBuy(key == 'i' || key == 'c'))
 			FailBuy();
 		else
 		{
-			Buy(key == 'i');
+			Buy(key == 'i' || key == 'c');
 			player.UpdateCargoCapacities();
 		}
 	}
-	else if(key == 's' || toCargo)
+	else if(key == 's' || toStorage)
 	{
-		if(!CanSell(toCargo))
-			FailSell(toCargo);
+		if(!CanSell(toStorage))
+			FailSell(toStorage);
 		else
 		{
 			int modifier = CanSellMultiple() ? Modifier() : 1;
-			for(int i = 0; i < modifier && CanSell(toCargo); ++i)
-				Sell(toCargo);
+			for(int i = 0; i < modifier && CanSell(toStorage); ++i)
+				Sell(toStorage);
 			player.UpdateCargoCapacities();
 		}
 	}
 	else if(key == SDLK_LEFT)
 	{
-		if(dragMain)
+		if(activePane != ShopPane::Sidebar)
 			MainLeft();
 		else
 			SideSelect(-1);
@@ -580,7 +640,7 @@ bool ShopPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, boo
 	}
 	else if(key == SDLK_RIGHT)
 	{
-		if(dragMain)
+		if(activePane != ShopPane::Sidebar)
 			MainRight();
 		else
 			SideSelect(1);
@@ -588,7 +648,7 @@ bool ShopPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, boo
 	}
 	else if(key == SDLK_UP)
 	{
-		if(dragMain)
+		if(activePane != ShopPane::Sidebar)
 			MainUp();
 		else
 			SideSelect(-4);
@@ -596,7 +656,7 @@ bool ShopPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, boo
 	}
 	else if(key == SDLK_DOWN)
 	{
-		if(dragMain)
+		if(activePane != ShopPane::Sidebar)
 			MainDown();
 		else
 			SideSelect(4);
@@ -606,6 +666,10 @@ bool ShopPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, boo
 		return DoScroll(Screen::Bottom());
 	else if(key == SDLK_PAGEDOWN)
 		return DoScroll(Screen::Top());
+	else if(key == SDLK_HOME)
+		return SetScrollToTop();
+	else if(key == SDLK_END)
+		return SetScrollToBottom();
 	else if(key >= '0' && key <= '9')
 	{
 		int group = key - '0';
@@ -619,15 +683,15 @@ bool ShopPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, boo
 			bool allWereSelected = true;
 			for(Ship *ship : added)
 				allWereSelected &= playerShips.erase(ship);
-			
+
 			if(allWereSelected)
 				added.clear();
-			
+
 			const System *here = player.GetSystem();
 			for(Ship *ship : added)
 				if(CanShowInSidebar(*ship, here))
 					playerShips.insert(ship);
-			
+
 			if(!playerShips.count(playerShip))
 				playerShip = playerShips.empty() ? nullptr : *playerShips.begin();
 		}
@@ -636,32 +700,34 @@ bool ShopPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command, boo
 			// Change the selection to the desired ships, if they are landed here.
 			playerShips.clear();
 			set<Ship *> wanted = player.GetGroup(group);
-			
+
 			const System *here = player.GetSystem();
 			for(Ship *ship : wanted)
 				if(CanShowInSidebar(*ship, here))
 					playerShips.insert(ship);
-			
+
 			if(!playerShips.count(playerShip))
 				playerShip = playerShips.empty() ? nullptr : *playerShips.begin();
 		}
 	}
+	else if(key == SDLK_TAB)
+		activePane = (activePane == ShopPane::Main ? ShopPane::Sidebar : ShopPane::Main);
 	else
 		return false;
-	
+
 	return true;
 }
 
 
 
-bool ShopPanel::Click(int x, int y, int clicks)
+bool ShopPanel::Click(int x, int y, int /* clicks */)
 {
 	dragShip = nullptr;
 	// Handle clicks on the buttons.
 	char button = CheckButton(x, y);
 	if(button)
 		return DoKey(button);
-	
+
 	// Check for clicks in the scroll arrows.
 	if(x >= Screen::Right() - 20)
 	{
@@ -677,12 +743,12 @@ bool ShopPanel::Click(int x, int y, int clicks)
 		if(y >= Screen::Bottom() - 20)
 			return Scroll(0, -4);
 	}
-	
-	Point point(x, y);
-	
+
+	const Point clickPoint(x, y);
+
 	// Check for clicks in the category labels.
 	for(const ClickZone<string> &zone : categoryZones)
-		if(zone.Contains(point))
+		if(zone.Contains(clickPoint))
 		{
 			bool toggleAll = (SDL_GetModState() & KMOD_SHIFT);
 			auto it = collapsed.find(zone.Value());
@@ -713,11 +779,11 @@ bool ShopPanel::Click(int x, int y, int clicks)
 			}
 			return true;
 		}
-	
+
 	// Handle clicks anywhere else by checking if they fell into any of the
 	// active click zones (main panel or side panel).
 	for(const Zone &zone : zones)
-		if(zone.Contains(point))
+		if(zone.Contains(clickPoint))
 		{
 			if(zone.GetShip())
 			{
@@ -730,19 +796,19 @@ bool ShopPanel::Click(int x, int y, int clicks)
 						SideSelect(dragShip);
 						return true;
 					}
-				
+
 				selectedShip = zone.GetShip();
 			}
 			else
 				selectedOutfit = zone.GetOutfit();
-			
+
 			// Scroll details into view in Step() when the height is known.
 			scrollDetailsIntoView = true;
 			mainDetailHeight = 0;
 			mainScroll = max(0., mainScroll + zone.ScrollY());
 			return true;
 		}
-	
+
 	return true;
 }
 
@@ -763,8 +829,12 @@ bool ShopPanel::Hover(int x, int y)
 		shipInfo.Hover(point);
 		outfitInfo.Hover(point);
 	}
-	
-	dragMain = (x < Screen::Right() - SIDE_WIDTH);
+
+	activePane = ShopPane::Main;
+	if(x > Screen::Right() - SIDEBAR_WIDTH)
+		activePane = ShopPane::Sidebar;
+	else if(x > Screen::Right() - SIDE_WIDTH)
+		activePane = ShopPane::Info;
 	return true;
 }
 
@@ -774,6 +844,7 @@ bool ShopPanel::Drag(double dx, double dy)
 {
 	if(dragShip)
 	{
+		isDraggingShip = true;
 		dragPoint += Point(dx, dy);
 		for(const Zone &zone : zones)
 			if(zone.Contains(dragPoint))
@@ -798,7 +869,7 @@ bool ShopPanel::Drag(double dx, double dy)
 		scrollDetailsIntoView = false;
 		DoScroll(dy);
 	}
-	
+
 	return true;
 }
 
@@ -807,6 +878,7 @@ bool ShopPanel::Drag(double dx, double dy)
 bool ShopPanel::Release(int x, int y)
 {
 	dragShip = nullptr;
+	isDraggingShip = false;
 	return true;
 }
 
@@ -822,17 +894,18 @@ bool ShopPanel::Scroll(double dx, double dy)
 
 int64_t ShopPanel::LicenseCost(const Outfit *outfit) const
 {
-	// Don't require a license for an outfit that you have in cargo or that you
-	// just sold to the outfitter. (Otherwise, there would be no way to transfer
-	// a restricted plundered outfit between ships or from cargo to a ship.)
-	if(player.Cargo().Get(outfit) || player.Stock(outfit) > 0)
+	// If the player is attempting to install an outfit from cargo, storage, or that they just
+	// sold to the shop, then ignore its license requirement, if any. (Otherwise there
+	// would be no way to use or transfer license-restricted outfits between ships.)
+	if((player.Cargo().Get(outfit) && playerShip) || player.Stock(outfit) > 0 ||
+			(player.Storage() && player.Storage()->Get(outfit)))
 		return 0;
-	
+
 	const Sale<Outfit> &available = player.GetPlanet()->Outfitter();
-	
+
 	int64_t cost = 0;
 	for(const string &name : outfit->Licenses())
-		if(!player.GetCondition("license: " + name))
+		if(!player.Conditions().Get("license: " + name))
 		{
 			const Outfit *license = GameData::Outfits().Find(name + " License");
 			if(!license || !license->Cost() || !available.Has(license))
@@ -881,11 +954,49 @@ double ShopPanel::Zone::ScrollY() const
 
 bool ShopPanel::DoScroll(double dy)
 {
-	double &scroll = dragMain ? mainScroll : sideScroll;
-	const double &maximum = dragMain ? maxMainScroll : maxSideScroll;
-	
-	scroll = max(0., min(maximum, scroll - dy));
-	
+	double *scroll = &mainScroll;
+	double maximum = maxMainScroll;
+	if(activePane == ShopPane::Info)
+	{
+		scroll = &infobarScroll;
+		maximum = maxInfobarScroll;
+	}
+	else if(activePane == ShopPane::Sidebar)
+	{
+		scroll = &sidebarScroll;
+		maximum = maxSidebarScroll;
+	}
+
+	*scroll = max(0., min(maximum, *scroll - dy));
+
+	return true;
+}
+
+
+
+bool ShopPanel::SetScrollToTop()
+{
+	if(activePane == ShopPane::Info)
+		infobarScroll = 0.;
+	else if(activePane == ShopPane::Sidebar)
+		sidebarScroll = 0.;
+	else
+		mainScroll = 0.;
+
+	return true;
+}
+
+
+
+bool ShopPanel::SetScrollToBottom()
+{
+	if(activePane == ShopPane::Info)
+		infobarScroll = maxInfobarScroll;
+	else if(activePane == ShopPane::Sidebar)
+		sidebarScroll = maxSidebarScroll;
+	else
+		mainScroll = maxMainScroll;
+
 	return true;
 }
 
@@ -898,7 +1009,7 @@ void ShopPanel::SideSelect(int count)
 	for( ; it != player.Ships().end(); ++it)
 		if((*it).get() == playerShip)
 			break;
-	
+
 	// Bail out if there are no ships to choose from.
 	if(it == player.Ships().end())
 	{
@@ -906,11 +1017,11 @@ void ShopPanel::SideSelect(int count)
 		playerShip = player.Flagship();
 		if(playerShip)
 			playerShips.insert(playerShip);
-		
+
 		return;
 	}
-	
-	
+
+
 	const System *here = player.GetSystem();
 	if(count < 0)
 	{
@@ -919,7 +1030,7 @@ void ShopPanel::SideSelect(int count)
 			if(it == player.Ships().begin())
 				it = player.Ships().end();
 			--it;
-			
+
 			if(CanShowInSidebar(**it, here))
 				++count;
 		}
@@ -931,7 +1042,7 @@ void ShopPanel::SideSelect(int count)
 			++it;
 			if(it == player.Ships().end())
 				it = player.Ships().begin();
-			
+
 			if(CanShowInSidebar(**it, here))
 				--count;
 		}
@@ -945,7 +1056,7 @@ void ShopPanel::SideSelect(Ship *ship)
 {
 	bool shift = (SDL_GetModState() & KMOD_SHIFT);
 	bool control = (SDL_GetModState() & (KMOD_CTRL | KMOD_GUI));
-	
+
 	if(shift)
 	{
 		bool on = false;
@@ -955,13 +1066,13 @@ void ShopPanel::SideSelect(Ship *ship)
 			// Skip any ships that are "absent" for whatever reason.
 			if(!CanShowInSidebar(*other, here))
 				continue;
-			
+
 			if(other.get() == ship || other.get() == playerShip)
 				on = !on;
 			else if(on)
 				playerShips.insert(other.get());
 		}
-	}	
+	}
 	else if(!control)
 		playerShips.clear();
 	else if(playerShips.count(ship))
@@ -971,7 +1082,7 @@ void ShopPanel::SideSelect(Ship *ship)
 			playerShip = playerShips.empty() ? nullptr : *playerShips.begin();
 		return;
 	}
-	
+
 	playerShip = ship;
 	playerShips.insert(playerShip);
 	sameSelectedTopY = true;
@@ -984,18 +1095,18 @@ void ShopPanel::MainLeft()
 	vector<Zone>::const_iterator start = MainStart();
 	if(start == zones.end())
 		return;
-	
+
 	vector<Zone>::const_iterator it = Selected();
 	// Special case: nothing is selected. Go to the last item.
 	if(it == zones.end())
 	{
 		--it;
-		mainScroll += it->Center().Y() - start->Center().Y();
+		mainScroll = maxMainScroll;
 		selectedShip = it->GetShip();
 		selectedOutfit = it->GetOutfit();
 		return;
 	}
-	
+
 	if(it == start)
 	{
 		mainScroll = 0;
@@ -1021,16 +1132,17 @@ void ShopPanel::MainRight()
 	vector<Zone>::const_iterator start = MainStart();
 	if(start == zones.end())
 		return;
-	
+
 	vector<Zone>::const_iterator it = Selected();
 	// Special case: nothing is selected. Select the first item.
 	if(it == zones.end())
 	{
+		// Already at mainScroll = 0, no scrolling needed.
 		selectedShip = start->GetShip();
 		selectedOutfit = start->GetOutfit();
 		return;
 	}
-	
+
 	int previousY = it->Center().Y();
 	++it;
 	if(it == zones.end())
@@ -1043,6 +1155,8 @@ void ShopPanel::MainRight()
 	{
 		if(it->Center().Y() != previousY)
 			mainScroll += it->Center().Y() - previousY - mainDetailHeight;
+		if(mainScroll > maxMainScroll)
+			mainScroll = maxMainScroll;
 		selectedShip = it->GetShip();
 		selectedOutfit = it->GetOutfit();
 	}
@@ -1055,25 +1169,25 @@ void ShopPanel::MainUp()
 	vector<Zone>::const_iterator start = MainStart();
 	if(start == zones.end())
 		return;
-	
+
 	vector<Zone>::const_iterator it = Selected();
 	// Special case: nothing is selected. Go to the last item.
 	if(it == zones.end())
 	{
 		--it;
-		mainScroll += it->Center().Y() - start->Center().Y();
+		mainScroll = maxMainScroll;
 		selectedShip = it->GetShip();
 		selectedOutfit = it->GetOutfit();
 		return;
 	}
-	
+
 	int previousX = it->Center().X();
 	int previousY = it->Center().Y();
 	while(it != start && it->Center().Y() == previousY)
 		--it;
 	while(it != start && it->Center().X() > previousX)
 		--it;
-	
+
 	if(it == start && it->Center().Y() == previousY)
 	{
 		mainScroll = 0;
@@ -1097,16 +1211,17 @@ void ShopPanel::MainDown()
 	vector<Zone>::const_iterator start = MainStart();
 	if(start == zones.end())
 		return;
-	
+
 	vector<Zone>::const_iterator it = Selected();
 	// Special case: nothing is selected. Select the first item.
 	if(it == zones.end())
 	{
+		// Already at mainScroll = 0, no scrolling needed.
 		selectedShip = start->GetShip();
 		selectedOutfit = start->GetOutfit();
 		return;
 	}
-	
+
 	int previousX = it->Center().X();
 	int previousY = it->Center().Y();
 	while(it != zones.end() && it->Center().Y() == previousY)
@@ -1118,13 +1233,13 @@ void ShopPanel::MainDown()
 		selectedOutfit = nullptr;
 		return;
 	}
-	
+
 	int newY = it->Center().Y();
 	while(it != zones.end() && it->Center().X() <= previousX && it->Center().Y() == newY)
 		++it;
 	--it;
-	
-	mainScroll += it->Center().Y() - previousY - mainDetailHeight;
+
+	mainScroll = min(mainScroll + it->Center().Y() - previousY - mainDetailHeight, maxMainScroll);
 	selectedShip = it->GetShip();
 	selectedOutfit = it->GetOutfit();
 }
@@ -1138,7 +1253,7 @@ vector<ShopPanel::Zone>::const_iterator ShopPanel::Selected() const
 	for( ; it != zones.end(); ++it)
 		if(it->GetShip() == selectedShip && it->GetOutfit() == selectedOutfit)
 			break;
-	
+
 	return it;
 }
 
@@ -1151,7 +1266,7 @@ vector<ShopPanel::Zone>::const_iterator ShopPanel::MainStart() const
 	vector<Zone>::const_iterator start = zones.begin();
 	while(start != zones.end() && start->Center().X() > margin)
 		++start;
-	
+
 	return start;
 }
 
@@ -1161,19 +1276,24 @@ vector<ShopPanel::Zone>::const_iterator ShopPanel::MainStart() const
 // letter of the button (or ' ' if it's not on a button).
 char ShopPanel::CheckButton(int x, int y)
 {
-	if(x < Screen::Right() - SIDE_WIDTH || y < Screen::Bottom() - BUTTON_HEIGHT)
+	if(x < Screen::Right() - SIDEBAR_WIDTH || y < Screen::Bottom() - BUTTON_HEIGHT)
 		return '\0';
-	
+
 	if(y < Screen::Bottom() - 40 || y >= Screen::Bottom() - 10)
 		return ' ';
-	
-	x -= Screen::Right() - SIDE_WIDTH;
+
+	x -= Screen::Right() - SIDEBAR_WIDTH;
 	if(x > 9 && x < 70)
-		return 'b';
+	{
+		if(!IsAlreadyOwned())
+			return 'b';
+		else
+			return 'i';
+	}
 	else if(x > 89 && x < 150)
 		return 's';
 	else if(x > 169 && x < 240)
 		return 'l';
-	
+
 	return ' ';
 }
